import json
import os

from jelm.core.jelm_class import Jelm
from jelm.core.io import reads_json, read_json


def test_json_reads():

    test_dic = {
        'metadata': {'author': 'me'},
        'objects': []
    }

    dump = json.dumps(test_dic)

    jelm = reads_json(dump)

    assert isinstance(jelm, Jelm)


def test_json_read(tmp_path):
    d = tmp_path / "sub"
    d.mkdir()
    p = d / "fing.jelm"

    test_dic = {
        'metadata': {'author': 'me'},
        'objects': []
    }

    dump = json.dumps(test_dic)

    p.write_text(dump)

    fp = os.fspath(p)

    el = read_json(fp)

    assert isinstance(el, Jelm)



import json

from .jelm_class import Jelm


def reads_json(dump: str) -> Jelm:

    return Jelm(**json.loads(dump))


def read_json(file_path: str) -> Jelm:

    with open(file_path) as fp:
        dump = fp.read()

    return reads_json(dump)



import pytest

from jelm import Jelm, Node, Edge


def test_init():

    el = Jelm(metadata={'author': 'John Doe'},
              objects=[])

    with pytest.raises(ValueError):
        el2 = Jelm(bad_kwarg="fing")

    assert isinstance(el.objects, list)
    assert isinstance(el.metadata, dict)


def test_get():

    el = Jelm()

    assert isinstance(el.get_dict(), dict)
    assert isinstance(el.get_json(), str)


def test_add_object():

    el = Jelm()

    el.add_object({'type': 'edge',
                     'source': 'n1',
                     'target': 'n2'})

    el.add_object({'type': 'node',
                     'id': 'n1'})

    el.add_object(Node(id='n2'))

    el.add_object(Node(id='n3',
                       attributes={'priority': 'low'}))

    with pytest.raises(ValueError):
        el.add_object({'no': 'type'})

    with pytest.raises(ValueError):
        el.add_object({'type': 'wrong'})

    el.add_edge('n3', 'n2')

    el.add_node('n4', {'order': 'latest'})


def test_iter():

    el = Jelm(metadata={'author': 'John Doe'},
              objects=[{'type': 'node',
                        'id': 'n1'},
                       {'type': 'node',
                        'id': 'n2'},
                       {'type': 'edge',
                        'source': 'n1',
                        'target': 'n2'}]
              )

    for idx, o in enumerate(el):
        if idx < 2:
            assert isinstance(o, Node)
        else:
            assert isinstance(o, Edge)



from setuptools import find_packages, setup
import jelm

pkg_name = 'jelm'

version = jelm.__version__


with open("README.md") as fp:
    long_description = fp.read()

if __name__ == '__main__':
    setup(
        name=pkg_name,
        version=version,
        description="python graph description",
        long_description=long_description,
        long_description_content_type="text/markdown",
        license='MIT',
        classifiers=[
            "License :: OSI Approved :: MIT License",
        ],
        url='https://github.com/endremborza/{}'.format(pkg_name),
        keywords='network graph json edgelist framework',
        author='Endre Márk Borza',
        author_email='endremborza@gmail.com',
        packages=find_packages(),
        include_package_data=True,
        python_requires='>=3.6',
    )
